@@ -1,4 +1,3 @@
-// [[Rcpp::plugins(cpp11)]]
 #include <Rcpp.h>
 using namespace Rcpp;
 
@@ -16,6 +15,14 @@ NumericVector fbstatstemp(NumericVector x, bool ext = false, int ng = 0, Integer
 
    if(!ext) {
     if(ng == 0) { // No groups
+      if(l == 1) { // need this so that qsu(1) works properly
+        NumericVector result = NumericVector::create(1,x[0],NA_REAL,x[0],x[0]);
+        if(setn) {
+          Rf_namesgets(result, CharacterVector::create("N","Mean","SD","Min","Max"));
+          Rf_classgets(result, CharacterVector::create("qsu","table"));
+        }
+        return result;
+      }
       int j = l-1;
       // double n = 0, min = R_PosInf, max = R_NegInf;
       // long double mean = 0, d1 = 0, M2 = 0;
@@ -45,9 +52,11 @@ NumericVector fbstatstemp(NumericVector x, bool ext = false, int ng = 0, Integer
               if(max < d1) max = d1;
               ++k;
             }
-            n = k;
-            mean = double(sum / n);
-            M2 = (double)sqrt((sq_sum - pow(mean,2)*n)/(n-1));
+            sum /= k;
+            sq_sum -= sum*sum*k;
+            M2 = (double)sqrt(sq_sum/(k-1));
+            n = (double)k;
+            mean = (double)sum;
           }
         } else mean = M2 = min = max = NA_REAL;
       } else { // with weights
@@ -967,7 +976,6 @@ SEXP fbstatslCpp(const List& x, bool ext = false, int ng = 0, const IntegerVecto
 //
 
 
-// // [[Rcpp::plugins(cpp11)]]
 // #include <Rcpp.h>
 // #include <unordered_set>
 // using namespace Rcpp;
