@@ -13,6 +13,9 @@
 static inline void iswap(int *a, int *b)           {int     tmp=*a; *a=*b; *b=tmp;}
 static inline void dswap(double *a, double *b)     {double  tmp=*a; *a=*b; *b=tmp;}
 
+// For weighted quantile methods
+static double eps = 10 * DBL_EPSILON;
+
 // Barebones quickselect algorithm from Numerical Recipes in C
 #undef QUICKSELECT
 #define QUICKSELECT(SWAP)                                                         \
@@ -280,10 +283,16 @@ SEXP fquantileC(SEXP x, SEXP Rprobs, SEXP w, SEXP o, SEXP Rnarm, SEXP Rtype, SEX
 
   if(asLogical(Rnames)) {
     SEXP names = PROTECT(allocVector(STRSXP, np)); ++nprotect;
-    char namei[5];
-    for(int i = 0; i < np; ++i) {
-      snprintf(namei, 5, "%d%%", (int)(probs[i]*100));
-      SET_STRING_ELT(names, i, mkChar(namei));
+    char namei[5], nameid[7];
+    for(int i = 0, dig; i < np; ++i) {
+      dig = (int)(probs[i]*1000) % 10;
+      if(dig == 0) {
+        snprintf(namei, 5, "%d%%", (int)(probs[i]*100));
+        SET_STRING_ELT(names, i, mkChar(namei));
+      } else {
+        snprintf(nameid, 7, "%d.%d%%", (int)(probs[i]*100), dig);
+        SET_STRING_ELT(names, i, mkChar(nameid));
+      }
     }
     namesgets(res, names);
   }
@@ -519,14 +528,15 @@ int k = 1;                                                                 \
 if(ret < 3) { /* lower (2), or average (1) element*/                       \
   while(wsum < h) wsum += pw[po[k++]];                                     \
   a = px[po[k-1]];                                                         \
-  if(ret == 2 || wsum != h) return a; /* h = sumw * Q must be > 0 here */  \
+  if(ret == 2 || wsum > h+eps) return a;/* h = sumw * Q must be > 0 here */\
   wsum = 2.0; wb = px[po[k]];                                              \
   while(pw[po[k]] == 0.0) { /* l should never be reached, I tested it */   \
     wb += px[po[++k]]; ++wsum;                                             \
   }                                                                        \
   return (a + wb) / wsum;                                                  \
 }                                                                          \
-while(wsum <= h) wsum += pw[po[k++]];                                      \
+wb = h + eps;                                                              \
+while(wsum <= wb) wsum += pw[po[k++]];                                      \
 a = px[po[k-1]];                                                           \
 if(ret == 3 || k == l || h == 0.0)                                         \
   return a;                                                                \
@@ -548,7 +558,7 @@ int k = 1;                                                                   \
 if(ret < 3) { /* lower (2), or average (1) element*/                         \
   while(wsum < h) wsum += pw[i_cc[k++]];                                     \
   a = x_cc[k-1];                                                             \
-  if(ret == 2 || wsum != h) res = a; /* h = sumw * Q must be > 0 here */     \
+  if(ret == 2 || wsum > h+eps) res = a; /* h = sumw * Q must be > 0 here */  \
   else {                                                                     \
     wsum = 2.0; wb = x_cc[k];                                                \
     while(pw[i_cc[k]] == 0.0) { /* n should never be reached, I tested it */ \
@@ -557,7 +567,8 @@ if(ret < 3) { /* lower (2), or average (1) element*/                         \
     res = (a + wb) / wsum;                                                   \
   }                                                                          \
 } else {                                                                     \
-  while(wsum <= h) wsum += pw[i_cc[k++]];                                    \
+  wb = h+eps;                                                                \
+  while(wsum <= wb) wsum += pw[i_cc[k++]];                                   \
   a = x_cc[k-1];                                                             \
   if(ret == 3 || k == n || h == 0.0) {                                       \
     res = a;                                                                 \
