@@ -43,10 +43,13 @@ void DFcopyAttr(SEXP out, SEXP x, int ng) {
 }
 
 // Faster than rep_len(value, n) and slightly faster than matrix(value, n) (which in turn is faster than rep_len)...
-SEXP falloc(SEXP value, SEXP n) {
-  int l = asInteger(n), tval = TYPEOF(value);
-  if(length(value) > 1) error("Must supply a single value to alloc()");
-  SEXP out = PROTECT(allocVector(tval, l));
+SEXP falloc(SEXP value, SEXP n, SEXP simplify)  {
+  int l = asInteger(n), tval = TYPEOF(value), isat = isVectorAtomic(value);
+  if((length(value) > 1 && isat) || asLogical(simplify) == 0) {
+    isat = 0;
+    tval = VECSXP;
+  }
+  SEXP out = PROTECT(allocVector(isat ? tval : VECSXP, l));
   switch(tval) {
     case INTSXP:
     case LGLSXP: {
@@ -57,7 +60,7 @@ SEXP falloc(SEXP value, SEXP n) {
     }
     case REALSXP: {
       double val = asReal(value), *pout = REAL(out);
-      if(val == 0.0) memset(pout, 0.0, l*sizeof(double));
+      if(val == 0.0) memset(pout, 0, l*sizeof(double));
       else for(int i = 0; i != l; ++i) pout[i] = val;
       break;
     }
@@ -66,14 +69,23 @@ SEXP falloc(SEXP value, SEXP n) {
       for(int i = 0; i != l; ++i) pout[i] = val;
       break;
     }
-    case VECSXP: {
+    case CPLXSXP: {
+      Rcomplex val = asComplex(value), *pout = COMPLEX(out);
+      for(int i = 0; i != l; ++i) pout[i] = val;
+      break;
+    }
+    case RAWSXP: {
+      Rbyte val = RAW(value)[0], *pout = RAW(out);
+      for(int i = 0; i != l; ++i) pout[i] = val;
+      break;
+    }
+    default: {
       SEXP *pout = SEXPPTR(out);
       for(int i = 0; i != l; ++i) pout[i] = value;
       break;
     }
-    default: error("Not supportd SEXP Type in alloc()");
   }
-  copyMostAttrib(value, out);
+  if(isat) copyMostAttrib(value, out);
   UNPROTECT(1);
   return out;
 }
@@ -91,9 +103,9 @@ SEXP groups2GRP(SEXP x, SEXP lx, SEXP gs) {
   // we take the R API (INTEGER()[i], REAL()[i], etc) outside loops for the simple types even when not parallel. For this
   // type list case (VECSXP) it might be that some items are ALTREP for example, so we really should use the heavier
   // _ELT accessor (VECTOR_ELT) inside the loop in this case.
-  SEXP *px = SEXPPTR(x);
+  const SEXP *px = SEXPPTR_RO(x);
 
-  for(int j = l; j--; ) { // This can go in any direction..
+  for(int j = 0; j != l; ++j) { // This can go in any direction..
     // SEXP column = VECTOR_ELT(x, j);
     int *pcolumn = INTEGER(px[j]), jp = j+1;
     for(int i = pgs[j]; i--; ) pout[pcolumn[i]] = jp; // This can go in any direction...
@@ -107,7 +119,7 @@ SEXP lassign(SEXP x, SEXP s, SEXP rows, SEXP fill) {
   int l = length(x), tr = TYPEOF(rows), ss = asInteger(s), rs = LENGTH(rows);
   SEXP out = PROTECT(allocVector(VECSXP, l));
   // SEXP *px = VECTOR_PTR(x); // -> Depreciated interface: https://github.com/hadley/r-internals/blob/ea892fa79bbffe961e78dbe9c90ce4ca3bf2d9bc/vectors.md
-  SEXP *px = SEXPPTR(x);
+  const SEXP *px = SEXPPTR_RO(x);
   double dfill = asReal(fill);
 
   if(tr == INTSXP) {
@@ -222,10 +234,11 @@ SEXP vlabels(SEXP x, SEXP attrn, SEXP usenam) {
     return labx;
   }
   SEXP res = PROTECT(allocVector(STRSXP, l));
-  SEXP *pres = STRING_PTR(res), *px = SEXPPTR(x);
+  SEXP *pres = STRING_PTR(res);
+  const SEXP *px = SEXPPTR_RO(x);
   for(int i = 0; i < l; ++i) {
     SEXP labxi = getAttrib(px[i], sym_attrn);
-    pres[i] = labxi == R_NilValue ? NA_STRING : STRING_ELT(labxi, 0);
+    pres[i] = TYPEOF(labxi) == STRSXP ? STRING_ELT(labxi, 0) : labxi == R_NilValue ? NA_STRING : asChar(labxi);
   }
   if(asLogical(usenam)) {
     SEXP nam = getAttrib(x, R_NamesSymbol);
@@ -236,12 +249,14 @@ SEXP vlabels(SEXP x, SEXP attrn, SEXP usenam) {
 }
 
 // Note: ind can be NULL...
+// TODO: option to shallow copy attributes list for setLabels / relabel !!!!
+
 SEXP setvlabels(SEXP x, SEXP attrn, SEXP value, SEXP ind) { // , SEXP sc
  if(!isString(attrn)) error("'attrn' must be of mode character");
  if(length(attrn) != 1) error("exactly one attribute 'attrn' must be given");
  if(TYPEOF(x) != VECSXP) error("X must be a list");
  int nprotect = 1, l = length(x), tv = TYPEOF(value); // , scl = asLogical(sc);
- SEXP *px = SEXPPTR(x); // , xsc;
+ const SEXP *px = SEXPPTR_RO(x); // , xsc;
  // if(scl) { // Create shallow copy
  //   if(INHERITS(x, char_datatable)) {
  //     xsc = PROTECT(Calloccol(x));
@@ -251,13 +266,13 @@ SEXP setvlabels(SEXP x, SEXP attrn, SEXP value, SEXP ind) { // , SEXP sc
  //   ++nprotect;
  //   px = SEXPPTR(xsc);
  // }
- SEXP *pv = px;
+ const SEXP *pv = px;
  if(tv != NILSXP) {
    if(tv == VECSXP || tv == STRSXP) {
-    pv = SEXPPTR(value);
+    pv = SEXPPTR_RO(value);
    } else {
     SEXP vl = PROTECT(coerceVector(value, VECSXP));
-    pv = SEXPPTR(vl); ++nprotect;
+    pv = SEXPPTR_RO(vl); ++nprotect;
    }
  }
  SEXP sym_attrn = PROTECT(installChar(STRING_ELT(attrn, 0)));
@@ -301,11 +316,6 @@ SEXP setvlabels(SEXP x, SEXP attrn, SEXP value, SEXP ind) { // , SEXP sc
 }
 
 
-SEXP setnames(SEXP x, SEXP nam) {
-  setAttrib(x, R_NamesSymbol, nam);
-  return x;
-}
-
 SEXP Cissorted(SEXP x, SEXP strictly) {
    return ScalarLogical(FALSE == isUnsorted(x, (Rboolean)asLogical(strictly)));
 }
@@ -543,4 +553,37 @@ SEXP all_funs(SEXP x) {
   return data.ans;
 }
 
+SEXP fnrowC(SEXP x) {
+  if(TYPEOF(x) == VECSXP) return ScalarInteger(length(x) ? length(VECTOR_ELT(x, 0)) : 0);
+  SEXP dim = getAttrib(x, R_DimSymbol);
+  if(TYPEOF(dim) != INTSXP) return R_NilValue;
+  return ScalarInteger(INTEGER(dim)[0]);
+}
 
+// Taken from: https://github.com/r-lib/rlang/blob/main/src/internal/env.c
+#define CLP_FRAME_LOCK_MASK (1 << 14)
+#define CLP_FRAME_IS_LOCKED(e) (ENVFLAGS(e) & CLP_FRAME_LOCK_MASK)
+#define CLP_UNLOCK_FRAME(e) SET_ENVFLAGS(e, ENVFLAGS(e) & (~CLP_FRAME_LOCK_MASK))
+
+SEXP unlock_collapse_namespace(SEXP env) {
+  if(TYPEOF(env) != ENVSXP) error("Unsupported object passed to C_unlock_collapse_namespace: %s", type2char(TYPEOF(env)));
+  CLP_UNLOCK_FRAME(env);
+  R_unLockBinding(install(".FAST_STAT_FUN_EXT"), env);
+  R_unLockBinding(install(".FAST_STAT_FUN_POLD"), env);
+  R_unLockBinding(install(".FAST_FUN_MOPS"), env);
+  R_unLockBinding(install(".COLLAPSE_ALL_EXPORTS"), env);
+  return CLP_FRAME_IS_LOCKED(env) == 0 ? ScalarLogical(1) : ScalarLogical(0);
+}
+
+
+// SEXP dot_return(SEXP x) {
+//   SEXP nam = getAttrib(x, R_NamesSymbol);
+//   if(TYPEOF(nam) == STRSXP) {
+//     int l = length(nam);
+//     SEXP *pnam = STRING_PTR(nam);
+//     for (int i = 0; i < l; ++i) if(length(pnam[i]) == 0) pnam[i] = 0;
+//
+//     substitute(x, R_NilValue);
+//
+//   }
+// }
