@@ -77,12 +77,6 @@ wwithin <- function(x, w, na.rm = FALSE, mean = 0) {
     return(x - wm + mean)
 }
 
-wBY <- function(x, f, FUN, w, ...) {
-  if(is.atomic(x) && !is.array(x)) return(unlist(Map(FUN, split(x, f), split(w, f), ...), use.names = FALSE))
-  wspl <- split(w, f)
-  if(is.atomic(x)) return(dapply(x, function(xi) unlist(Map(FUN, split(xi, f), wspl, ...), use.names = FALSE)))
-  qDF(dapply(x, function(xi) unlist(Map(FUN, split(xi, f), wspl, ...), use.names = FALSE), return = "matrix"))
-}
 
 # fbetween
 
@@ -180,18 +174,18 @@ test_that("fbetween with weights performs like wbetween (defined above)", {
   expect_equal(fbetween(mtcars, w = wdat, na.rm = FALSE), dapply(mtcars, wbetween, wdat))
   expect_equal(fbetween(mtcNA, w = wdat, na.rm = FALSE), dapply(mtcNA, wbetween, wdat))
   expect_equal(fbetween(mtcNA, w = wdat), dapply(mtcNA, wbetween, wdat, na.rm = TRUE))
-  expect_equal(fbetween(x, f, w), wBY(x, f, wbetween, w))
-  expect_equal(fbetween(x, f, w, na.rm = FALSE), wBY(x, f, wbetween, w))
-  expect_equal(fbetween(xNA, f, w, na.rm = FALSE), wBY(xNA, f, wbetween, w))
-  expect_equal(fbetween(xNA, f, w), wBY(xNA, f, wbetween, w, na.rm = TRUE))
-  expect_equal(fbetween(m, g, wdat), wBY(m, g, wbetween, wdat))
-  expect_equal(fbetween(m, g, wdat, na.rm = FALSE), wBY(m, g, wbetween, wdat))
-  expect_equal(fbetween(mNA, g, wdat, na.rm = FALSE),  wBY(mNA, g, wbetween, wdat))
-  expect_equal(fbetween(mNA, g, wdat), wBY(mNA, g, wbetween, wdat, na.rm = TRUE))
-  expect_equal(fbetween(mtcars, g, wdat), wBY(mtcars, g, wbetween, wdat))
-  expect_equal(fbetween(mtcars, g, wdat, na.rm = FALSE), wBY(mtcars, g, wbetween, wdat))
-  expect_equal(fbetween(mtcNA, g, wdat, na.rm = FALSE), wBY(mtcNA, g, wbetween, wdat))
-  expect_equal(fbetween(mtcNA, g, wdat), wBY(mtcNA, g, wbetween, wdat, na.rm = TRUE))
+  expect_equal(fbetween(x, f, w), BY(x, f, wbetween, w))
+  expect_equal(fbetween(x, f, w, na.rm = FALSE), BY(x, f, wbetween, w))
+  expect_equal(fbetween(xNA, f, w, na.rm = FALSE), BY(xNA, f, wbetween, w))
+  expect_equal(fbetween(xNA, f, w), BY(xNA, f, wbetween, w, na.rm = TRUE))
+  expect_equal(fbetween(m, g, wdat), BY(m, g, wbetween, wdat))
+  expect_equal(fbetween(m, g, wdat, na.rm = FALSE), BY(m, g, wbetween, wdat))
+  expect_equal(fbetween(mNA, g, wdat, na.rm = FALSE),  BY(mNA, g, wbetween, wdat))
+  expect_equal(fbetween(mNA, g, wdat), BY(mNA, g, wbetween, wdat, na.rm = TRUE))
+  expect_equal(fbetween(mtcars, g, wdat), BY(mtcars, g, wbetween, wdat))
+  expect_equal(fbetween(mtcars, g, wdat, na.rm = FALSE), BY(mtcars, g, wbetween, wdat))
+  expect_equal(fbetween(mtcNA, g, wdat, na.rm = FALSE), BY(mtcNA, g, wbetween, wdat))
+  expect_equal(fbetween(mtcNA, g, wdat), BY(mtcNA, g, wbetween, wdat, na.rm = TRUE))
   # missing weights
   expect_equal(fbetween(NA, w = NA), wbetween(NA, NA))
   expect_equal(fbetween(NA, w = NA, na.rm = FALSE), wbetween(NA, NA))
@@ -214,18 +208,18 @@ test_that("fbetween with weights performs like wbetween (defined above)", {
   expect_equal(fbetween(mtcars, w = wdatNA, na.rm = FALSE), dapply(mtcars, wbetween, wdatNA))
   expect_equal(fbetween(mtcNA, w = wdatNA, na.rm = FALSE), dapply(mtcNA, wbetween, wdatNA))
   expect_equal(fbetween(mtcNA, w = wdatNA), dapply(mtcNA, wbetween, wdatNA, na.rm = TRUE))
-  expect_equal(fbetween(x, f, wNA), wBY(x, f, wbetween, wNA, na.rm = TRUE))
-  expect_equal(fbetween(x, f, wNA, na.rm = FALSE), wBY(x, f, wbetween, wNA))
-  expect_equal(fbetween(xNA, f, wNA, na.rm = FALSE), wBY(xNA, f, wbetween, wNA))
-  expect_equal(fbetween(xNA, f, wNA), wBY(xNA, f, wbetween, wNA, na.rm = TRUE))
-  expect_equal(fbetween(m, g, wdatNA), wBY(m, g, wbetween, wdatNA, na.rm = TRUE))
-  expect_equal(fbetween(m, g, wdatNA, na.rm = FALSE), wBY(m, g, wbetween, wdatNA))
-  expect_equal(fbetween(mNA, g, wdatNA, na.rm = FALSE),  wBY(mNA, g, wbetween, wdatNA))
-  expect_equal(fbetween(mNA, g, wdatNA), wBY(mNA, g, wbetween, wdatNA, na.rm = TRUE))
-  expect_equal(fbetween(mtcars, g, wdatNA), wBY(mtcars, g, wbetween, wdatNA, na.rm = TRUE))
-  expect_equal(fbetween(mtcars, g, wdatNA, na.rm = FALSE), wBY(mtcars, g, wbetween, wdatNA))
-  expect_equal(fbetween(mtcNA, g, wdatNA, na.rm = FALSE), wBY(mtcNA, g, wbetween, wdatNA))
-  expect_equal(fbetween(mtcNA, g, wdatNA), wBY(mtcNA, g, wbetween, wdatNA, na.rm = TRUE))
+  expect_equal(fbetween(x, f, wNA), BY(x, f, wbetween, wNA, na.rm = TRUE))
+  expect_equal(fbetween(x, f, wNA, na.rm = FALSE), BY(x, f, wbetween, wNA))
+  expect_equal(fbetween(xNA, f, wNA, na.rm = FALSE), BY(xNA, f, wbetween, wNA))
+  expect_equal(fbetween(xNA, f, wNA), BY(xNA, f, wbetween, wNA, na.rm = TRUE))
+  expect_equal(fbetween(m, g, wdatNA), BY(m, g, wbetween, wdatNA, na.rm = TRUE))
+  expect_equal(fbetween(m, g, wdatNA, na.rm = FALSE), BY(m, g, wbetween, wdatNA))
+  expect_equal(fbetween(mNA, g, wdatNA, na.rm = FALSE),  BY(mNA, g, wbetween, wdatNA))
+  expect_equal(fbetween(mNA, g, wdatNA), BY(mNA, g, wbetween, wdatNA, na.rm = TRUE))
+  expect_equal(fbetween(mtcars, g, wdatNA), BY(mtcars, g, wbetween, wdatNA, na.rm = TRUE))
+  expect_equal(fbetween(mtcars, g, wdatNA, na.rm = FALSE), BY(mtcars, g, wbetween, wdatNA))
+  expect_equal(fbetween(mtcNA, g, wdatNA, na.rm = FALSE), BY(mtcNA, g, wbetween, wdatNA))
+  expect_equal(fbetween(mtcNA, g, wdatNA), BY(mtcNA, g, wbetween, wdatNA, na.rm = TRUE))
 })
 
 test_that("fbetween performs numerically stable", {
@@ -669,18 +663,18 @@ test_that("fwithin with weights performs like wwithin (defined above)", {
   expect_equal(fwithin(mtcars, w = wdat, na.rm = FALSE), dapply(mtcars, wwithin, wdat))
   expect_equal(fwithin(mtcNA, w = wdat, na.rm = FALSE), dapply(mtcNA, wwithin, wdat))
   expect_equal(fwithin(mtcNA, w = wdat), dapply(mtcNA, wwithin, wdat, na.rm = TRUE))
-  expect_equal(fwithin(x, f, w), wBY(x, f, wwithin, w))
-  expect_equal(fwithin(x, f, w, na.rm = FALSE), wBY(x, f, wwithin, w))
-  expect_equal(fwithin(xNA, f, w, na.rm = FALSE), wBY(xNA, f, wwithin, w))
-  expect_equal(fwithin(xNA, f, w), wBY(xNA, f, wwithin, w, na.rm = TRUE))
-  expect_equal(fwithin(m, g, wdat), wBY(m, g, wwithin, wdat))
-  expect_equal(fwithin(m, g, wdat, na.rm = FALSE), wBY(m, g, wwithin, wdat))
-  expect_equal(fwithin(mNA, g, wdat, na.rm = FALSE),  wBY(mNA, g, wwithin, wdat))
-  expect_equal(fwithin(mNA, g, wdat), wBY(mNA, g, wwithin, wdat, na.rm = TRUE))
-  expect_equal(fwithin(mtcars, g, wdat), wBY(mtcars, g, wwithin, wdat))
-  expect_equal(fwithin(mtcars, g, wdat, na.rm = FALSE), wBY(mtcars, g, wwithin, wdat))
-  expect_equal(fwithin(mtcNA, g, wdat, na.rm = FALSE), wBY(mtcNA, g, wwithin, wdat))
-  expect_equal(fwithin(mtcNA, g, wdat), wBY(mtcNA, g, wwithin, wdat, na.rm = TRUE))
+  expect_equal(fwithin(x, f, w), BY(x, f, wwithin, w))
+  expect_equal(fwithin(x, f, w, na.rm = FALSE), BY(x, f, wwithin, w))
+  expect_equal(fwithin(xNA, f, w, na.rm = FALSE), BY(xNA, f, wwithin, w))
+  expect_equal(fwithin(xNA, f, w), BY(xNA, f, wwithin, w, na.rm = TRUE))
+  expect_equal(fwithin(m, g, wdat), BY(m, g, wwithin, wdat))
+  expect_equal(fwithin(m, g, wdat, na.rm = FALSE), BY(m, g, wwithin, wdat))
+  expect_equal(fwithin(mNA, g, wdat, na.rm = FALSE),  BY(mNA, g, wwithin, wdat))
+  expect_equal(fwithin(mNA, g, wdat), BY(mNA, g, wwithin, wdat, na.rm = TRUE))
+  expect_equal(fwithin(mtcars, g, wdat), BY(mtcars, g, wwithin, wdat))
+  expect_equal(fwithin(mtcars, g, wdat, na.rm = FALSE), BY(mtcars, g, wwithin, wdat))
+  expect_equal(fwithin(mtcNA, g, wdat, na.rm = FALSE), BY(mtcNA, g, wwithin, wdat))
+  expect_equal(fwithin(mtcNA, g, wdat), BY(mtcNA, g, wwithin, wdat, na.rm = TRUE))
   # missing weights
   expect_equal(fwithin(NA, w = NA), wwithin(NA, NA))
   expect_equal(fwithin(NA, w = NA, na.rm = FALSE), wwithin(NA, NA))
@@ -703,18 +697,18 @@ test_that("fwithin with weights performs like wwithin (defined above)", {
   expect_equal(fwithin(mtcars, w = wdatNA, na.rm = FALSE), dapply(mtcars, wwithin, wdatNA))
   expect_equal(fwithin(mtcNA, w = wdatNA, na.rm = FALSE), dapply(mtcNA, wwithin, wdatNA))
   expect_equal(fwithin(mtcNA, w = wdatNA), dapply(mtcNA, wwithin, wdatNA, na.rm = TRUE))
-  expect_equal(fwithin(x, f, wNA), wBY(x, f, wwithin, wNA, na.rm = TRUE))
-  expect_equal(fwithin(x, f, wNA, na.rm = FALSE), wBY(x, f, wwithin, wNA))
-  expect_equal(fwithin(xNA, f, wNA, na.rm = FALSE), wBY(xNA, f, wwithin, wNA))
-  expect_equal(fwithin(xNA, f, wNA), wBY(xNA, f, wwithin, wNA, na.rm = TRUE))
-  expect_equal(fwithin(m, g, wdatNA), wBY(m, g, wwithin, wdatNA, na.rm = TRUE))
-  expect_equal(fwithin(m, g, wdatNA, na.rm = FALSE), wBY(m, g, wwithin, wdatNA))
-  expect_equal(fwithin(mNA, g, wdatNA, na.rm = FALSE),  wBY(mNA, g, wwithin, wdatNA))
-  expect_equal(fwithin(mNA, g, wdatNA), wBY(mNA, g, wwithin, wdatNA, na.rm = TRUE))
-  expect_equal(fwithin(mtcars, g, wdatNA), wBY(mtcars, g, wwithin, wdatNA, na.rm = TRUE))
-  expect_equal(fwithin(mtcars, g, wdatNA, na.rm = FALSE), wBY(mtcars, g, wwithin, wdatNA))
-  expect_equal(fwithin(mtcNA, g, wdatNA, na.rm = FALSE), wBY(mtcNA, g, wwithin, wdatNA))
-  expect_equal(fwithin(mtcNA, g, wdatNA), wBY(mtcNA, g, wwithin, wdatNA, na.rm = TRUE))
+  expect_equal(fwithin(x, f, wNA), BY(x, f, wwithin, wNA, na.rm = TRUE))
+  expect_equal(fwithin(x, f, wNA, na.rm = FALSE), BY(x, f, wwithin, wNA))
+  expect_equal(fwithin(xNA, f, wNA, na.rm = FALSE), BY(xNA, f, wwithin, wNA))
+  expect_equal(fwithin(xNA, f, wNA), BY(xNA, f, wwithin, wNA, na.rm = TRUE))
+  expect_equal(fwithin(m, g, wdatNA), BY(m, g, wwithin, wdatNA, na.rm = TRUE))
+  expect_equal(fwithin(m, g, wdatNA, na.rm = FALSE), BY(m, g, wwithin, wdatNA))
+  expect_equal(fwithin(mNA, g, wdatNA, na.rm = FALSE),  BY(mNA, g, wwithin, wdatNA))
+  expect_equal(fwithin(mNA, g, wdatNA), BY(mNA, g, wwithin, wdatNA, na.rm = TRUE))
+  expect_equal(fwithin(mtcars, g, wdatNA), BY(mtcars, g, wwithin, wdatNA, na.rm = TRUE))
+  expect_equal(fwithin(mtcars, g, wdatNA, na.rm = FALSE), BY(mtcars, g, wwithin, wdatNA))
+  expect_equal(fwithin(mtcNA, g, wdatNA, na.rm = FALSE), BY(mtcNA, g, wwithin, wdatNA))
+  expect_equal(fwithin(mtcNA, g, wdatNA), BY(mtcNA, g, wwithin, wdatNA, na.rm = TRUE))
 })
 
 test_that("fwithin with custom mean and weights performs like wwithin (defined above)", {
@@ -732,18 +726,18 @@ test_that("fwithin with custom mean and weights performs like wwithin (defined a
   expect_equal(fwithin(mtcars, w = wdat, na.rm = FALSE, mean = 4.8456), dapply(mtcars, wwithin, wdat, mean = 4.8456))
   expect_equal(fwithin(mtcNA, w = wdat, na.rm = FALSE, mean = 4.8456), dapply(mtcNA, wwithin, wdat, mean = 4.8456))
   expect_equal(fwithin(mtcNA, w = wdat, mean = 4.8456), dapply(mtcNA, wwithin, wdat, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(x, f, w, mean = 4.8456), wBY(x, f, wwithin, w, mean = 4.8456))
-  expect_equal(fwithin(x, f, w, na.rm = FALSE, mean = 4.8456), wBY(x, f, wwithin, w, mean = 4.8456))
-  expect_equal(fwithin(xNA, f, w, na.rm = FALSE, mean = 4.8456), wBY(xNA, f, wwithin, w, mean = 4.8456))
-  expect_equal(fwithin(xNA, f, w, mean = 4.8456), wBY(xNA, f, wwithin, w, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(m, g, wdat, mean = 4.8456), wBY(m, g, wwithin, wdat, mean = 4.8456))
-  expect_equal(fwithin(m, g, wdat, na.rm = FALSE, mean = 4.8456), wBY(m, g, wwithin, wdat, mean = 4.8456))
-  expect_equal(fwithin(mNA, g, wdat, na.rm = FALSE, mean = 4.8456),  wBY(mNA, g, wwithin, wdat, mean = 4.8456))
-  expect_equal(fwithin(mNA, g, wdat, mean = 4.8456), wBY(mNA, g, wwithin, wdat, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(mtcars, g, wdat, mean = 4.8456), wBY(mtcars, g, wwithin, wdat, mean = 4.8456))
-  expect_equal(fwithin(mtcars, g, wdat, na.rm = FALSE, mean = 4.8456), wBY(mtcars, g, wwithin, wdat, mean = 4.8456))
-  expect_equal(fwithin(mtcNA, g, wdat, na.rm = FALSE, mean = 4.8456), wBY(mtcNA, g, wwithin, wdat, mean = 4.8456))
-  expect_equal(fwithin(mtcNA, g, wdat, mean = 4.8456), wBY(mtcNA, g, wwithin, wdat, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(x, f, w, mean = 4.8456), BY(x, f, wwithin, w, mean = 4.8456))
+  expect_equal(fwithin(x, f, w, na.rm = FALSE, mean = 4.8456), BY(x, f, wwithin, w, mean = 4.8456))
+  expect_equal(fwithin(xNA, f, w, na.rm = FALSE, mean = 4.8456), BY(xNA, f, wwithin, w, mean = 4.8456))
+  expect_equal(fwithin(xNA, f, w, mean = 4.8456), BY(xNA, f, wwithin, w, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(m, g, wdat, mean = 4.8456), BY(m, g, wwithin, wdat, mean = 4.8456))
+  expect_equal(fwithin(m, g, wdat, na.rm = FALSE, mean = 4.8456), BY(m, g, wwithin, wdat, mean = 4.8456))
+  expect_equal(fwithin(mNA, g, wdat, na.rm = FALSE, mean = 4.8456),  BY(mNA, g, wwithin, wdat, mean = 4.8456))
+  expect_equal(fwithin(mNA, g, wdat, mean = 4.8456), BY(mNA, g, wwithin, wdat, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(mtcars, g, wdat, mean = 4.8456), BY(mtcars, g, wwithin, wdat, mean = 4.8456))
+  expect_equal(fwithin(mtcars, g, wdat, na.rm = FALSE, mean = 4.8456), BY(mtcars, g, wwithin, wdat, mean = 4.8456))
+  expect_equal(fwithin(mtcNA, g, wdat, na.rm = FALSE, mean = 4.8456), BY(mtcNA, g, wwithin, wdat, mean = 4.8456))
+  expect_equal(fwithin(mtcNA, g, wdat, mean = 4.8456), BY(mtcNA, g, wwithin, wdat, na.rm = TRUE, mean = 4.8456))
   # missing weights
   expect_equal(fwithin(x, w = wNA, mean = 4.8456), wwithin(x, wNA, na.rm = TRUE, mean = 4.8456))
   expect_equal(fwithin(x, w = wNA, na.rm = FALSE, mean = 4.8456), wwithin(x, wNA, mean = 4.8456))
@@ -758,18 +752,18 @@ test_that("fwithin with custom mean and weights performs like wwithin (defined a
   expect_equal(fwithin(mtcars, w = wdatNA, na.rm = FALSE, mean = 4.8456), dapply(mtcars, wwithin, wdatNA, mean = 4.8456))
   expect_equal(fwithin(mtcNA, w = wdatNA, na.rm = FALSE, mean = 4.8456), dapply(mtcNA, wwithin, wdatNA, mean = 4.8456))
   expect_equal(fwithin(mtcNA, w = wdatNA, mean = 4.8456), dapply(mtcNA, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(x, f, wNA, mean = 4.8456), wBY(x, f, wwithin, wNA, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(x, f, wNA, na.rm = FALSE, mean = 4.8456), wBY(x, f, wwithin, wNA, mean = 4.8456))
-  expect_equal(fwithin(xNA, f, wNA, na.rm = FALSE, mean = 4.8456), wBY(xNA, f, wwithin, wNA, mean = 4.8456))
-  expect_equal(fwithin(xNA, f, wNA, mean = 4.8456), wBY(xNA, f, wwithin, wNA, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(m, g, wdatNA, mean = 4.8456), wBY(m, g, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(m, g, wdatNA, na.rm = FALSE, mean = 4.8456), wBY(m, g, wwithin, wdatNA, mean = 4.8456))
-  expect_equal(fwithin(mNA, g, wdatNA, na.rm = FALSE, mean = 4.8456),  wBY(mNA, g, wwithin, wdatNA, mean = 4.8456))
-  expect_equal(fwithin(mNA, g, wdatNA, mean = 4.8456), wBY(mNA, g, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(mtcars, g, wdatNA, mean = 4.8456), wBY(mtcars, g, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
-  expect_equal(fwithin(mtcars, g, wdatNA, na.rm = FALSE, mean = 4.8456), wBY(mtcars, g, wwithin, wdatNA, mean = 4.8456))
-  expect_equal(fwithin(mtcNA, g, wdatNA, na.rm = FALSE, mean = 4.8456), wBY(mtcNA, g, wwithin, wdatNA, mean = 4.8456))
-  expect_equal(fwithin(mtcNA, g, wdatNA, mean = 4.8456), wBY(mtcNA, g, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(x, f, wNA, mean = 4.8456), BY(x, f, wwithin, wNA, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(x, f, wNA, na.rm = FALSE, mean = 4.8456), BY(x, f, wwithin, wNA, mean = 4.8456))
+  expect_equal(fwithin(xNA, f, wNA, na.rm = FALSE, mean = 4.8456), BY(xNA, f, wwithin, wNA, mean = 4.8456))
+  expect_equal(fwithin(xNA, f, wNA, mean = 4.8456), BY(xNA, f, wwithin, wNA, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(m, g, wdatNA, mean = 4.8456), BY(m, g, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(m, g, wdatNA, na.rm = FALSE, mean = 4.8456), BY(m, g, wwithin, wdatNA, mean = 4.8456))
+  expect_equal(fwithin(mNA, g, wdatNA, na.rm = FALSE, mean = 4.8456),  BY(mNA, g, wwithin, wdatNA, mean = 4.8456))
+  expect_equal(fwithin(mNA, g, wdatNA, mean = 4.8456), BY(mNA, g, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(mtcars, g, wdatNA, mean = 4.8456), BY(mtcars, g, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
+  expect_equal(fwithin(mtcars, g, wdatNA, na.rm = FALSE, mean = 4.8456), BY(mtcars, g, wwithin, wdatNA, mean = 4.8456))
+  expect_equal(fwithin(mtcNA, g, wdatNA, na.rm = FALSE, mean = 4.8456), BY(mtcNA, g, wwithin, wdatNA, mean = 4.8456))
+  expect_equal(fwithin(mtcNA, g, wdatNA, mean = 4.8456), BY(mtcNA, g, wwithin, wdatNA, na.rm = TRUE, mean = 4.8456))
 })
 
 test_that("Weighted centering on overall weighted mean performs as indended", {
